import { Request, Response } from 'express';
import { resolveAddress } from './geocoding';
import { Address, AvailableTimeslots } from './types';
import { randomUUID } from 'crypto';
// DOME 
import { getAvailableTimeSlots } from './services/timeslotsService';
import { getHolidays } from './services/holidaysService';

const env = require('dotenv');

// create a hashing for caching delivery slots. this needs to be a caching db.
const deliveriesCache = new Map();
// create a hashing for caching timeslots. this needs to be a caching db.
const slotsInUse = new Map();

export const resolveAddressHandler = (req: Request, res: Response) => {
  if (!req.body.searchTerm) {
    res.status(400).json({ error: 'Missing searchTerm' });
    return;
  }
  const address: Promise<Address> = resolveAddress(req.body.searchTerm);
  address.then((result) => {
    res.status(200).json(result);
  })
};

export const timeslotsHandler = async (req: Request, res: Response) => {
  if (!req.body.address) {
    res.status(400).json({ error: 'Missing address' });
    return;
  }
  const address: Address = req.body.address;
  const timeSlots: AvailableTimeslots[] = await availableTimeSlots(address);
  const availableTimeSlotsResult: AvailableTimeslots[] = await filterOutHolidaysByCountryCode(address, timeSlots);

  availableTimeSlotsResult.forEach((timeSlot, index) => {
    if (slotsInUse.has(timeSlot.id) && slotsInUse.get(timeSlot.id).length >= 2) {
      availableTimeSlotsResult.splice(index, 1);
    }
  });
  res.status(200).json(availableTimeSlotsResult);
};

export const deliveriesHandler = (req: Request, res: Response) => {
  const userId = req.body.userId;
  const slotId = req.body.timeslotId;

  // Idea: validate userId and slotId needed
  if (!userId || !slotId) {
    res.status(400).json({ error: 'Missing userId or slotId' });
    return;
  }

  // Idea: check if user has already booked a delivery
  // if (deliveriesCache.has(userId) && deliveriesCache.get(userId).slotId === slotId) {
  //   res.status(400).json({ error: 'User has already booked a delivery' });
  //   return;
  // }

  // Idea: check if timeslot is already full in cache
  if (slotsInUse.has(slotId) && slotsInUse.get(slotId).length >= 2) {
    res.status(400).json({ error: 'This timeslot is already full' });
    return;
  }
  const deliveryId = randomUUID();

  // Idea: create new delivery for user
  const delivery = {
    _id: deliveryId,
    userId,
    slotId,
    deliveryCreatedDate: new Date()
  };
  if (slotsInUse.has(slotId)) {
    slotsInUse.get(slotId).push(deliveryId);
  } else {
    slotsInUse.set(slotId, [deliveryId]);
  }
  deliveriesCache.set(deliveryId, delivery);

  console.log("deliveriesCache", deliveriesCache)
  console.log("slotsInUse", slotsInUse)
  res.status(200).json(delivery);
};


export const cancelDeliveryHandler = (req: Request, res: Response) => {
  // TODO: Implement cancel delivery functionality
  // DELETE /deliveries/:deliveryId
  const deliveryId = req.params.deliveryId;

  if (!deliveryId) {
    res.status(400).json({ error: 'Missing deliveryId' });
    return;
  }

  // remove from deliveriesCache and extract slotId
  const delivery = deliveriesCache.get(deliveryId);
  if (!delivery) {
    res.status(400).json({ error: 'Delivery not found' });
    return;
  }
  const slotId = delivery.slotId;
  deliveriesCache.delete(deliveryId);
  // remove from slotsInUse by slotId
  const slot = slotsInUse.get(slotId);
  const index = slot.indexOf(deliveryId);
  if (index > -1) {
    slot.splice(index, 1);
  }
  res.status(200).json(delivery);
};

export const dailyDeliveriesHandler = (req: Request, res: Response) => {
  // TODO: Implement daily deliveries functionality
};

export const weeklyDeliveriesHandler = (req: Request, res: Response) => {
  // TODO: Implement weekly deliveries functionality
};



async function filterOutHolidaysByCountryCode(address: Address, availableTimeSlot: AvailableTimeslots[]): Promise<AvailableTimeslots[]> {
  const countryCode = address.code;
  const holidays = await getHolidays();

  const filteredAvailableTimeSlot = [];

  console.log(availableTimeSlot)
  holidays.forEach((holiday) => {
    if (holiday.country === countryCode) {
      availableTimeSlot.forEach((timeSlot) => {
        if (timeSlot.start_time.split(' ')[0] !== holiday.date) {
          filteredAvailableTimeSlot.push(timeSlot)
        }
      })
    }
  });
  return filteredAvailableTimeSlot;
}


async function availableTimeSlots(address: Address) {
  const availableTimeSlot = [];
  const timeslots = await getAvailableTimeSlots();
  // check by postcode if any available timeslots
  for (const timeslot of timeslots.courier_available_timeslots) {
    if (timeslot.supported_postcodes.includes(address.postcode)) {
      availableTimeSlot.push({
        id: timeslot.id,
        start_time: timeslot.start_time,
        end_time: timeslot.end_time
      });
    }
  }
  return availableTimeSlot;
}